@@ -183,6 +183,19 @@ region_free (Region   *region,
 	}
 }
 
+static void
+set_max_line_count (GitgDiffView *view,
+                    guint         max_line_count)
+{
+	view->priv->max_line_count = max_line_count;
+
+	if (view->priv->line_renderer)
+	{
+		gitg_diff_line_renderer_set_max_line_count (view->priv->line_renderer,
+		                                            max_line_count);
+	}
+}
+
 static void
 regions_free (GitgDiffView *view)
 {
@@ -194,7 +207,8 @@ regions_free (GitgDiffView *view)
 	view->priv->regions = NULL;
 	view->priv->last_region = NULL;
 	view->priv->last_scan_line = 0;
-	view->priv->max_line_count = 99;
+
+	set_max_line_count (view, 99);
 }
 
 static void
@@ -445,18 +459,6 @@ enable_diff_view (GitgDiffView *view)
 	                          GTK_SOURCE_GUTTER_RENDERER (view->priv->line_renderer),
 	                          0);
 
-	/*gtk_source_gutter_set_cell_data_func (gutter,
-	                                      GTK_SOURCE_GUTTER_RENDERER (view->priv->line_renderer),
-	                                      (GtkSourceGutterDataFunc)line_renderer_data_func,
-	                                      view,
-	                                      NULL);
-
-	gtk_source_gutter_set_cell_size_func (gutter,
-	                                      GTK_SOURCE_GUTTER_RENDERER (view->priv->line_renderer),
-	                                      (GtkSourceGutterSizeFunc)line_renderer_size_func,
-	                                      view,
-	                                      NULL);*/
-
 	g_signal_connect (view->priv->line_renderer,
 	                  "query-data",
 	                  (GCallback) line_renderer_query_data_cb,
@@ -511,7 +513,7 @@ ensure_max_line (GitgDiffView *view, Hunk *hunk)
 
 	if (m > view->priv->max_line_count)
 	{
-		view->priv->max_line_count = m;
+		set_max_line_count (view, m);
 
 		gtk_source_gutter_queue_draw (gtk_source_view_get_gutter (GTK_SOURCE_VIEW (view), GTK_TEXT_WINDOW_LEFT));
 	}
@@ -769,27 +771,6 @@ get_initial_counters (GitgDiffView *view, Region *region, guint line, guint coun
 			++counters[1];
 	}
 }
-/*
-static void
-line_renderer_size_func (GtkSourceGutter *gutter,
-                         GtkCellRenderer *cell,
-                         GitgDiffView    *view)
-{
-	g_object_set (cell,
-	              "line_old", view->priv->max_line_count,
-	              "line_new", view->priv->max_line_count,
-	              NULL);
-
-	if (view->priv->label_func)
-	{
-		gchar *label = view->priv->label_func (view,
-		                                       -1,
-		                                       view->priv->label_func_user_data);
-
-		g_object_set (cell, "label", label, NULL);
-		g_free (label);
-	}
-}*/
 
 static void
 line_renderer_query_data_cb (GtkSourceGutterRenderer      *renderer,
