@@ -485,16 +485,11 @@ free_cached_header (gpointer header)
 static void
 free_cached_headers (GitgRevisionChangesPanel *changes_panel)
 {
-	g_slist_foreach (changes_panel->priv->cached_headers,
-	                 (GFunc)free_cached_header,
-	                 NULL);
-
-	g_slist_free (changes_panel->priv->cached_headers);
+	g_slist_free_full (changes_panel->priv->cached_headers, free_cached_header);
 
 	changes_panel->priv->cached_headers = NULL;
 }
 
-
 static void
 gitg_revision_changes_panel_finalize (GObject *object)
 {
@@ -704,7 +699,7 @@ on_diff_files_begin_loading (GitgShell                *shell,
 	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (self->priv->diff_files)),
 	                       cursor);
 
-	gdk_cursor_unref (cursor);
+	g_object_unref (cursor);
 }
 
 static void
@@ -819,7 +814,7 @@ on_diff_begin_loading (GitgShell                *shell,
 	GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
 	gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET(self->priv->diff)),
 	                       cursor);
-	gdk_cursor_unref (cursor);
+	g_object_unref (cursor);
 }
 
 static void
