@@ -26,6 +26,7 @@
 #include "gitg-revision-panel.h"
 #include "gitg-stat-view.h"
 #include "gitg-uri.h"
+#include "gitg-avatar-cache.h"
 
 #include <glib/gi18n.h>
 #include <stdlib.h>
@@ -46,6 +47,7 @@ struct _GitgRevisionDetailsPanelPrivate
 	GtkLabel *committer;
 	GtkLabel *subject;
 	GtkTable *parents;
+	GtkImage *avatar;
 
 	GtkWidget *panel_widget;
 	GtkTextView *text_view;
@@ -60,6 +62,8 @@ struct _GitgRevisionDetailsPanelPrivate
 
 	GSList *stats;
 	GitgWindow *window;
+
+	GitgAvatarCache *cache;
 };
 
 static void gitg_revision_panel_iface_init (GitgRevisionPanelInterface *iface);
@@ -125,6 +129,7 @@ initialize_ui (GitgRevisionDetailsPanel *panel)
 	priv->subject = GTK_LABEL (gtk_builder_get_object (priv->builder, "label_subject"));
 	priv->parents = GTK_TABLE (gtk_builder_get_object (priv->builder, "table_parents"));
 	priv->text_view = GTK_TEXT_VIEW (gtk_builder_get_object (priv->builder, "text_view_details"));
+	priv->avatar = GTK_IMAGE (gtk_builder_get_object (priv->builder, "image_avatar"));
 
 	gchar const *lbls[] = {
 		"label_subject_lbl",
@@ -211,6 +216,12 @@ gitg_revision_details_panel_dispose (GObject *object)
 		panel->priv->shell = NULL;
 	}
 
+	if (panel->priv->cache)
+	{
+		g_object_unref (panel->priv->cache);
+		panel->priv->cache = NULL;
+	}
+
 	G_OBJECT_CLASS (gitg_revision_details_panel_parent_class)->dispose (object);
 }
 static void
@@ -548,6 +559,8 @@ gitg_revision_details_panel_init (GitgRevisionDetailsPanel *self)
 	                  "update",
 	                  G_CALLBACK (on_shell_update),
 	                  self);
+
+	self->priv->cache = gitg_avatar_cache_new ();
 }
 
 #define HASH_KEY "GitgRevisionDetailsPanelHashKey"
@@ -725,6 +738,117 @@ update_details (GitgRevisionDetailsPanel *panel)
 	g_free (sha1);
 }
 
+static void
+update_author (GitgRevisionDetailsPanel *panel)
+{
+	gchar const *author;
+	gchar const *author_email;
+
+	author = gitg_revision_get_author (panel->priv->revision);
+	author_email = gitg_revision_get_author_email (panel->priv->revision);
+
+	if (author == NULL || author_email == NULL ||
+	    *author == '\0' || *author_email == '\0')
+	{
+		gtk_label_set_text (panel->priv->author, "");
+	}
+	else
+	{
+		gchar *tmp;
+		gchar *date;
+
+		date = gitg_revision_get_author_date_for_display (panel->priv->revision);
+		tmp = g_markup_printf_escaped ("<a href='mailto:%s'>%s &lt;%s&gt;</a> (%s)",
+		                               author_email,
+		                               author,
+		                               author_email,
+		                               date);
+
+		gtk_label_set_markup (panel->priv->author, tmp);
+
+		g_free (tmp);
+		g_free (date);
+	}
+}
+
+static void
+update_committer (GitgRevisionDetailsPanel *panel)
+{
+	gchar const *committer;
+	gchar const *committer_email;
+
+	committer = gitg_revision_get_committer (panel->priv->revision);
+	committer_email = gitg_revision_get_committer_email (panel->priv->revision);
+
+	if (committer == NULL || committer_email == NULL ||
+	    *committer == '\0' || *committer_email == '\0')
+	{
+		gtk_label_set_text (panel->priv->committer, "");
+	}
+	else
+	{
+		gchar *tmp;
+		gchar *date;
+
+		date = gitg_revision_get_committer_date_for_display (panel->priv->revision);
+		tmp = g_markup_printf_escaped ("<a href='mailto:%s'>%s &lt;%s&gt;</a> (%s)",
+		                               committer_email,
+		                               committer,
+		                               committer_email,
+		                               date);
+
+		gtk_label_set_markup (panel->priv->committer, tmp);
+
+		g_free (tmp);
+		g_free (date);
+	}
+}
+
+static void
+avatar_ready (GObject                  *source_object,
+              GAsyncResult             *res,
+              GitgRevisionDetailsPanel *panel)
+{
+	GdkPixbuf *pixbuf;
+	GError *error = NULL;
+
+	pixbuf = gitg_avatar_cache_load_finish (panel->priv->cache,
+	                                        res,
+	                                        &error);
+
+	gtk_widget_set_visible (GTK_WIDGET (panel->priv->avatar),
+	                        error == NULL);
+
+	if (error == NULL)
+	{
+		gtk_image_set_from_pixbuf (panel->priv->avatar, pixbuf);
+	}
+}
+
+static void
+set_avatar (GitgRevisionDetailsPanel *panel,
+            const gchar              *email)
+{
+	if (email == NULL || *email == '\0')
+	{
+		gtk_widget_hide (GTK_WIDGET (panel->priv->avatar));
+	}
+	else
+	{
+		gchar *uri;
+
+		uri = gitg_avatar_cache_get_gravatar_uri (panel->priv->cache,
+		                                          email);
+		gitg_avatar_cache_load_uri_async (panel->priv->cache,
+		                                  uri,
+		                                  G_PRIORITY_DEFAULT,
+		                                  NULL,
+		                                  (GAsyncReadyCallback)avatar_ready,
+		                                  panel);
+		g_free (uri);
+	}
+}
+
 static void
 reload (GitgRevisionDetailsPanel *panel)
 {
@@ -734,47 +858,26 @@ reload (GitgRevisionDetailsPanel *panel)
 	if (panel->priv->revision)
 	{
 		gchar *tmp;
-		gchar *date;
 
-		date = gitg_revision_get_author_date_for_display (panel->priv->revision);
-		tmp = g_markup_printf_escaped ("<a href='mailto:%s'>%s &lt;%s&gt;</a> (%s)",
-		                               gitg_revision_get_author_email (panel->priv->revision),
-		                               gitg_revision_get_author (panel->priv->revision),
-		                               gitg_revision_get_author_email (panel->priv->revision),
-		                               date);
+		update_author (panel);
+		update_committer (panel);
 
-		gtk_label_set_markup (panel->priv->author, tmp);
+		tmp = g_markup_printf_escaped ("<b>%s</b>",
+		                               gitg_revision_get_subject (panel->priv->revision));
+
+		gtk_label_set_markup (panel->priv->subject, tmp);
 
 		g_free (tmp);
-		g_free (date);
 
-		date = gitg_revision_get_committer_date_for_display (panel->priv->revision);
-		tmp = g_markup_printf_escaped ("<a href='mailto:%s'>%s &lt;%s&gt;</a> (%s)",
-		                               gitg_revision_get_committer_email (panel->priv->revision),
-		                               gitg_revision_get_committer (panel->priv->revision),
-		                               gitg_revision_get_committer_email (panel->priv->revision),
-		                               date);
-
-		gtk_label_set_markup (panel->priv->committer, tmp);
-
-		g_free (tmp);
-		g_free (date);
-
-		gchar *subject;
-
-		subject = g_markup_printf_escaped ("<b>%s</b>",
-		                                   gitg_revision_get_subject (panel->priv->revision));
-
-		gtk_label_set_markup (panel->priv->subject, subject);
-		g_free (subject);
-
-		gchar *sha = gitg_revision_get_sha1 (panel->priv->revision);
-		gtk_label_set_text (panel->priv->sha, sha);
+		tmp = gitg_revision_get_sha1 (panel->priv->revision);
+		gtk_label_set_text (panel->priv->sha, tmp);
 
 		cb = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-		gtk_clipboard_set_text (cb, sha, -1);
+		gtk_clipboard_set_text (cb, tmp, -1);
 
-		g_free (sha);
+		g_free (tmp);
+
+		set_avatar (panel, gitg_revision_get_author_email (panel->priv->revision));
 	}
 	else
 	{
@@ -782,6 +885,7 @@ reload (GitgRevisionDetailsPanel *panel)
 		gtk_label_set_text (panel->priv->committer, "");
 		gtk_label_set_text (panel->priv->subject, "");
 		gtk_label_set_text (panel->priv->sha, "");
+		set_avatar (panel, NULL);
 	}
 
 	// Update parents
